#!/usr/bin/python
# -*- coding:utf-8 -*-
"""
Download TV program table from Yahoo.co.jp and convert it to csv file.

Usage : python loadtv2csv.py [ -area (prefecture) ]
                             [ -wave (am/fm/vhf/uhf/bs_digital) ]
                             [ -offset <number> ]
                             [ -o <output filename> ]
"""

import os, string, re, sys

class Program:
    """Television program containing time, title, description, genre and link.
    """
    # genres
    GENRE_NONE   = 0
    GENRE_FOOD   = 1
    GENRE_DRAMA  = 2
    GENRE_SPORTS = 3
    GENRE_ANIME  = 4
    GENRE_MUSIC  = 5
    GENRE_MOVIE  = 6

    # colors for genres
    COLOR_NONE   = '#ffffff'
    COLOR_FOOD   = '#ffcf9c'
    COLOR_DRAMA  = '#ffffce'
    COLOR_ANIME  = '#ddeeff'
    COLOR_SPORTS = '#eeddff'
    COLOR_MUSIC  = '#ddffdd'
    COLOR_MOVIE  = '#ffdddd'

    # static regular expressions
    regex_time = re.compile( "<b>(\\d\\d:\\d\\d)</b>" )
    regex_title = re.compile( "<a[^>]+href=([^>\\s]+)\\s*>(.*?)</a>" )
    regex_rmtag = re.compile( "<.*?>" )
    regex_img   = re.compile( "<img\\s.*?alt=['\"]?(.*?)['\"]?[\\s>][^>]*>" )

    def __init__( self, area, data, span ):
        data = data.strip()
        self.area = area
        self.comment = ""
        self.genre = self.GENRE_NONE

        mt = self.regex_time.search( data )
        self.time = mt.group( 1 )
        hr, mn = self.time.split( ':' )

        self.startTime = string.atoi( hr ) * 60 + string.atoi( mn )
        self.endTime = self.startTime + span

        ml = self.regex_title.search( data, mt.end() )
        self.title = ml.group( 2 )
        self.link = ml.group( 1 )

        self.gcode = ''
        # convert icons to abbreviations
        mi = self.regex_img.search( self.title )
        if mi: 
            while mi:
                alt = mi.group( 1 )
                if alt == 'ニュース'   : alt = '[Ｎ]'
                elif alt == '天気予報' : alt = '[天]'
                elif alt == '再放送'   : alt = '[再]'
                elif alt == '映画'   :
                    alt = ''
                    self.genre = self.GENRE_MOVIE
                    pass
                elif alt == '最終回':
                    alt = ''
                    self.comment = alt
                    pass
                elif alt == '前編' or alt == '中編' or alt == '後編':
                    self.comment = alt
                    alt = ''
                    pass
                self.title = self.title[ 0: mi.start() ] + alt + self.title[ mi.end(): ]
                mi = self.regex_img.search( self.title )
                pass
            pass

        self.description = self.regex_rmtag.sub( "", data[ ml.end(): ] ).strip()
        self.span = span
        self.idval = -1
        self.stacknumber = Program.counter
        Program.counter += 1
        pass

    # shared id
    counter = 0
    
    def setID( self, idnum ):
        self.idvalue = idnum
        pass

    def getID( self ):
        return self.idvalue

    def setGenre( self, genre ):
        self.genre = genre
        pass

    def setSpan( self, span ):
        if span < 0: span += 1440
        self.span = span
        self.endTime = self.startTime + span
        pass

    def offsetTime( self, minutes ):
        if self.startTime < 1440:
            self.startTime += minutes
            pass
        if self.endTime < 1440:
            self.endTime += minutes
            pass
        pass

    def setGCode( self, code ):
        self.gcode = code
        pass

    def __repr__( self ):
        address_epg = "http://rd.yahoo.co.jp/tv/iepg/tokyo/?http://tv.yahoo.co.jp/bin/iepg?id=%d&area=%s" % ( self.idvalue, self.area )
        output = "%d,%d,%s,%s,%s,%d,%s,%s" % ( self.startTime, self.span, self.title, self.description, self.comment, self.genre, self.gcode, address_epg )
        return output
    pass

class ProgramTable:
    """Programs of the day. The objects contains a sorted array of Porgram objects and
    name of the channel."""
    def __init__( self, progs = None ):
        self.name = ""
        if progs == None:
            self.programs = []
        else:
            self.programs = progs
        pass

    def set_name( self, name ):
        self.name = name
        pass

    def get_name( self ):
        return self.name

    def append( self, prog ):
        self.programs.append( prog )
        pass

    def find_program_at( self, time ):
        for p in self.programs:
            if p.startTime == time:
                return p
            elif p.startTime > time:
                return None
            pass
        return None

    def is_left( self, table ):
        for p in self.programs:
            q = table.find_program_at( p.startTime )
            if q:
                if p.stacknumber < q.stacknumber: return True
                return False
            pass
        return False
    pass

if __name__ == "__main__":
    import urllib, tempfile, time

    # read arguments
    i = 1
    use_urllib = False
    file_out = sys.stdout
    filename_output = None
    flag_radio = False
    area = 'tokyo'
    wave = 'vhf'
    offset = 0
    timeshift = ''
    while i < len( sys.argv ) - 1:
        ar = sys.argv[ i ]
        if ar == '-o':
            filename_output = sys.argv[ i + 1 ]
            pass
        elif ar == '-area':
            area = sys.argv[ i + 1 ]
        elif ar == '-wave':
            wave = sys.argv[ i + 1 ]
            pass
        elif ar == '-offset':
            try:
                offset = string.atoi( sys.argv[ i + 1 ] )
            except:
                pass
            pass
        i += 1
        pass
    
    if filename_output:
        file_out = open( filename_output, "w" )
        pass

    if wave == 'uhf' or wave == 'bs':
        baseurl = "http://tv.yahoo.co.jp/uhf_bs/%s/%s24.html?g=1"
    elif wave == 'bs_digital':
        baseurl = "http://tv.yahoo.co.jp/bs_digital/%s/%s24.html?g=1"
    elif wave == 'am':
        flag_radio = True
        baseurl = "http://tv.yahoo.co.jp/radio/%s/%s24.html"
    elif wave == 'fm':
        flag_radio = True
        baseurl = "http://tv.yahoo.co.jp/radio/%s/%s24fm.html"
    else:
        baseurl = url = "http://tv.yahoo.co.jp/vhf/%s/%s24.html?g=1"
        pass

    # laod HTML
    lt = time.localtime( time.time() + offset * 3600 * 24 )
    today = "%d%s%s" % ( lt[ 0 ], ( "0%d" % lt[ 1 ] )[ -2: ], ( "0%d" % lt[ 2 ] )[ -2: ] )
    url = baseurl % ( area, today )

    if use_urllib:
        stream = urllib.urlopen( url )
        fn_tmp = tempfile.mktemp()
        open( fn_tmp, "w" ).write( stream.read() )
        stream.close()
    else:
        fn_tmp = tempfile.mktemp()
        command = "/usr/bin/curl \"%s\" --silent -o %s" % ( url, fn_tmp )
        #print command
        os.system( command )
        pass
    
    fn_utf = tempfile.mktemp()
    os.system( "/usr/bin/iconv -f euc-jp -t utf-8 %s > %s" % ( fn_tmp, fn_utf ) )
    os.unlink( fn_tmp )

    # execute external script for radio programs
    if flag_radio:
        import r2c
        r2c.convert_radio_html_to_csv( fn_utf, file_out )
        file_out.close()
        os.unlink( fn_utf )
        sys.exit()
        pass

    # parse HTML
    fn = open( fn_utf )
    line = fn.readline()
    area_html = ""
    wave_html = ""
    date_html = ""
    # read title
    re_title = re.compile( "<title>(.*?)</title>" )
    while line != "":
        m = re_title.search( line )
        if m:
            title = m.group( 1 )
            area_html = title
            m = re.search( "\[(.*?)/(.*?)/[^\]]+\]", title )
            if m:
                area_html = m.group( 2 )
                wave_html = m.group( 1 )
                pass
            m = re.search( "(\\d+)月(\\d+)日", title )
            if m:
                date_html = m.group( 1 ) + "/" + m.group( 2 )
                pass
            break
        line = fn.readline()
        pass
    
    # read channel names
    channels = []
    channelurls = {}
    flag_start = False
    r_channel = re.compile( "<font color=''>(.*?)</font>" )
    r_link = re.compile( "href='(.*?)'" )

    while line != "":
        line = line.strip()
        if line == "<!--- OUTLINE_TABLE -->":
            line = fn.readline()
            break
        line = fn.readline()

    while line != "":
        line = line.strip()
        if line == "<!--- OUTLINE_TABLE -->": flag_start = True
        if flag_start:
            #print line
            if line.find( "</tr>" ) >= 0: break
            m_channel = r_channel.search( line )
            if m_channel:
                cn = re.sub( "<.*?>", "", m_channel.group( 1 ) )
                channels.append( cn )
                #print cn
                m_link = r_link.search( line )
                if m_link:
                    channelurls[ cn ] = m_link.group( 1 )
                    #print channelurls[ cn ]
                    pass
                pass
            pass
        line = fn.readline()
        pass
    
    # read programs
    programs = {}
    storedorder = []

    r_span = re.compile( "rowspan='?(\\d+)'?" )
    r_bgcol = re.compile( "bgcolor='(#[\0-9a-zA-Z]+)'" )
    r_prog = re.compile( "href=/bin/search\?id=(\\d+)" )
    #r_timeshift = None
    #if timeshift == '1': # G code
    r_timeshift = re.compile( "<small>\((\\d+)\)</small>" )
    #elif timeshift == '2': # iEPG
    #<a href="http://rd.yahoo.co.jp/tv/iepg/tokyo/?http://tv.yahoo.co.jp/bin/iepg?id=32569514&area=tokyo">
    #    r_timeshift = re.compile( "href=\"(http://rd.yahoo.co.jp/tv/iepg[^\"]+)" )
    #pass
    #r_radio = re.compile( "<td><small><b>\\d\\d:\\d\\d</b></small>.*?</td></tr>" )
    flag_midnight = False

    genre = Program.GENRE_NONE
    p = None
    while line != "":
        if line.find( "<small>0</small>" ) >= 0:
            flag_midnight = True
        m_span = r_span.search( line )
        if m_span:
            span = string.atoi( m_span.group( 1 ) )
            # Genre
            m_bgcol = r_bgcol.search( line )
            bgcol = None
            while m_bgcol:
                bgcol = m_bgcol.group( 1 )
                m_bgcol = r_bgcol.search( line, m_bgcol.end() )
                pass
            if bgcol:
                bgcol == string.lower( bgcol )
                if   bgcol == Program.COLOR_FOOD:
                    genre = Program.GENRE_FOOD
                elif bgcol == Program.COLOR_DRAMA:
                    genre = Program.GENRE_DRAMA
                elif bgcol == Program.COLOR_ANIME:
                    genre = Program.GENRE_ANIME
                elif bgcol == Program.COLOR_SPORTS:
                    genre = Program.GENRE_SPORTS
                elif bgcol == Program.COLOR_MUSIC:
                    genre = Program.GENRE_MUSIC
                elif bgcol == Program.COLOR_MOVIE:
                    genre = Program.GENRE_MOVIE
                else: genre = Program.GENRE_NONE
                p = None
            else:
                genre = Program.GENRE_NONE
                pass
            pass
        m_prog = r_prog.search( line )
        if m_prog:
            p = Program( area, line, span )
            p.setGenre( genre )
            if flag_midnight:
                p.offsetTime( 1440 )
                pass
            idval = string.atoi( m_prog.group( 1 ) )
            p.setID( idval )
            #p.setEPGAddress( 'http://rd.yahoo.co.jp/tv/iepg/tokyo/?http://tv.yahoo.co.jp/bin/iepg?%s&area=%s' % ( idval, area ) )
            programs[ idval ] = p
            pass
        elif r_timeshift and p:
            m_timeshift = r_timeshift.match( line )
            if m_timeshift:
                p.setGCode( m_timeshift.group( 1 ) )
                pass
            pass
        line = fn.readline()
        pass
    fn.close()
    
    keys = programs.keys()
    keys.sort()
    delims = []
    delims.append( 0 )
    for i in range( 1, len( keys ) ):
        p0 = programs[ keys[ i - 1 ] ]
        p1 = programs[ keys[ i ] ]
        t0 = p0.endTime
        t1 = p1.startTime
        if p0.startTime > t1:
            t1 += 1440
            pass
        if p0.startTime > p1.startTime:
            delims.append( i )
            pass
        elif t0 != t1:
            p0.setSpan( p1.startTime - p0.startTime )
            pass
        pass
    delims.append( len( keys ) )

    tables = []
    for d in range( 0, len( delims ) - 1 ):
        table = ProgramTable()
        for key in keys[ delims[ d ]: delims[ d + 1 ] ]:
            table.append( programs[ key ] )
            pass
        tables.append( table )
        pass


    for i in range( len( tables ) ):
        table_left = tables[ i ]
        pos_left = i
        for j in range( i + 1, len( tables ) ):
            if tables[ j ].is_left( table_left ):
                table_left = tables[ j ]
                pos_left = j
                pass
            pass
        tables[ pos_left ] = tables[ i ]
        tables[ i ] = table_left
        table_left.set_name( channels[ i ] )
        pass

    # output CSV file
    file_out.write( "#%s,%s,%s\n" % ( area_html, date_html, wave_html ) )
    for i in range( len( tables ) ):
        table = tables[ i ]
        ch = channels[ i ]
        if channelurls.has_key( ch ):
            file_out.write( "*%s,%s\n" % ( ch, channelurls[ ch ] ) )
        else:
            file_out.write( "*%s\n" % ch )
            pass
        for p in table.programs:
            file_out.write( "%s\n" % repr( p ) )
            pass
        pass
    file_out.close()
    os.unlink( fn_utf )
    pass

